/*
 * This file is part of pspcipher.
 *
 * Copyright (C) 2008 hrimfaxi (outmatch@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <pspsdk.h>
#include <pspkernel.h>
#include <pspthreadman_kernel.h>
#include <pspcrypt.h>
#include <pspdebug.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include "psputilsforkernel.h"
#include "systemctrl.h"
#include "../pspcipher.h"

u8 key_2DA8[16] = {
	0x80, 0x02, 0xC0, 0xBF, 0x00, 0x0A, 0xC0, 0xBF,
	0x40, 0x03, 0xC0, 0xBF, 0x40, 0x00, 0x00, 0x00
};

u8 key_2DB8[16] = {
	0x4C, 0x2D, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00,
	0xB8, 0x15, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

u8 key_2D90[16] = {
	0x55, 0x19, 0x35, 0x10, 0x48, 0xD8, 0x2E, 0x46,
	0xA8, 0xB1, 0x47, 0x77, 0xDC, 0x49, 0x6A, 0x6F
};

u8 key_2D80[16] = {
	0x40, 0x02, 0xC0, 0xBF, 0x20, 0x02, 0xC0, 0xBF,
	0x5C, 0x68, 0x2B, 0x95, 0x5F, 0x40, 0x7B, 0xB8
};

/* kernel modules 2.80 */
u8 key_2380[16] = {
	0xCA, 0xFB, 0xBF, 0xC7, 0x50, 0xEA, 0xB4, 0x40,
	0x8E, 0x44, 0x5C, 0x63, 0x53, 0xCE, 0x80, 0xB1
};

/* kernel modules 2.60-2.71 */
u8 key_2370[16] = {
	0xC3, 0x24, 0x89, 0xD3, 0x80, 0x87, 0xB2, 0x4E,
	0x4C, 0xD7, 0x49, 0xE4, 0x9D, 0x1D, 0x34, 0xD1
};

u8 key_21C0[16] = {
	0x6A, 0x19, 0x71, 0xF3, 0x18, 0xDE, 0xD3, 0xA2,
	0x6D, 0x3B, 0xDE, 0xC7, 0xBE, 0x98, 0xE2, 0x4C
};

u8 key_22E0[16] = {
	0x66, 0x0F, 0xCB, 0x3B, 0x30, 0x75, 0xE3, 0x10,
	0x0A, 0x95, 0x65, 0xC7, 0x3C, 0x93, 0x87, 0x22
};

u8 key_2250[16] = {
	0x50, 0xCC, 0x03, 0xAC, 0x3F, 0x53, 0x1A, 0xFA,
	0x0A, 0xA4, 0x34, 0x23, 0x86, 0x61, 0x7F, 0x97
};

/* 6.00-6.20 kernel and phat */
u8 keys620_0[0x10] = 
{
	0xD6, 0xBD, 0xCE, 0x1E, 0x12, 0xAF, 0x9A, 0xE6,
	0x69, 0x30, 0xDE, 0xDA, 0x88, 0xB8, 0xFF, 0xFB
};

/* 6.00-6.20 slim kernel */
u8 keys620_1[0x10] = 
{
	0x1D, 0x13, 0xE9, 0x50, 0x04, 0x73, 0x3D, 0xD2,
	0xE1, 0xDA, 0xB9, 0xC1, 0xE6, 0x7B, 0x25, 0xA7
};

u8 g_kernel_phat_key[0x10] = { 0 };
u8 g_kernel_slim_key[0x10] = { 0 };
u8 buf_1[0x150] = { 0 };
u8 buf_2[0xb4] = { 0 };
u8 buf_3[0x150] = { 0 };
u8 buf_4[0x90] = { 0 };

// blacklisted module list from FW 6.20, yes it's pretty useless :)
u8 blacklist_data[0x970] = {
	0xC2, 0x37, 0xA4, 0xC1, 0x9D, 0xD8, 0x61, 0x2E, 0x9D, 0xEF, 0xC7, 0xDE, 0x0F, 0xD2, 0xBA, 0x6B, 
	0xB7, 0xFD, 0x4F, 0xB4, 0xBD, 0xF3, 0x26, 0xEB, 0x0C, 0x36, 0x96, 0xF1, 0xB6, 0x64, 0x23, 0xE5, 
	0xB3, 0xB1, 0x28, 0x33, 0x9D, 0x2A, 0x86, 0xEF, 0x4E, 0x4D, 0x8E, 0xA4, 0x0B, 0x70, 0x30, 0x39, 
	0x95, 0x5F, 0x14, 0x76, 0x0F, 0xD2, 0xF0, 0xE0, 0xC6, 0x93, 0x22, 0x8F, 0x5D, 0x46, 0xE9, 0xD9, 
	0x68, 0x29, 0x4E, 0x7C, 0xA5, 0xD1, 0x6F, 0xDF, 0xFB, 0xE3, 0x04, 0x4D, 0x17, 0x11, 0x70, 0xE9, 
	0x2C, 0x71, 0x5B, 0xDB, 0x1C, 0xC9, 0xFF, 0xFE, 0x8E, 0x41, 0x1C, 0xD9, 0x13, 0x62, 0xEC, 0xEF, 
	0x18, 0x5A, 0xC0, 0x44, 0x94, 0x12, 0xB1, 0x5C, 0xA2, 0xC3, 0x80, 0x77, 0x31, 0xC9, 0x43, 0x4B, 
	0xA2, 0x40, 0xA0, 0x35, 0x18, 0x3D, 0xF4, 0x1B, 0xF7, 0x58, 0x61, 0x74, 0xBD, 0x88, 0xF3, 0x38, 
	0x35, 0xF6, 0x22, 0xD3, 0x0F, 0x78, 0x43, 0xC0, 0xD8, 0x80, 0x5D, 0x07, 0x5F, 0xCF, 0xB4, 0x3A, 
	0x0E, 0xF1, 0x06, 0x12, 0xE8, 0xB8, 0xBD, 0x92, 0x39, 0xF8, 0x99, 0x90, 0xD5, 0xD4, 0x74, 0x1E, 
	0x75, 0xB3, 0xB0, 0x43, 0xF1, 0xC5, 0xF3, 0x78, 0xB4, 0xCD, 0xB3, 0xD5, 0xE7, 0x22, 0x28, 0x2D, 
	0x4B, 0x97, 0xA9, 0xEA, 0x57, 0x82, 0xC6, 0xDC, 0x80, 0xBE, 0xAC, 0x60, 0xFA, 0x2F, 0x53, 0x49, 
	0xF8, 0xC4, 0x33, 0x52, 0xC8, 0x18, 0x52, 0xFA, 0x1D, 0x47, 0x06, 0x56, 0xEE, 0xB5, 0x53, 0xA4, 
	0xD7, 0x60, 0x9D, 0x95, 0x21, 0x8A, 0x20, 0xA2, 0xF5, 0xED, 0x80, 0xE6, 0x74, 0x7B, 0xD4, 0x49, 
	0x0E, 0xFA, 0xC0, 0xDE, 0x6C, 0x4C, 0x1D, 0xF9, 0xF4, 0x4F, 0x41, 0x59, 0xE7, 0xF7, 0xBE, 0xC2, 
	0xE6, 0xB2, 0x30, 0x61, 0x29, 0xE2, 0x9B, 0x49, 0xD9, 0xB5, 0x66, 0x77, 0x6A, 0x51, 0x4C, 0xC0, 
	0x8C, 0x50, 0x32, 0xBA, 0x23, 0x8E, 0xB6, 0xBB, 0x7E, 0x81, 0x7D, 0x4B, 0x3F, 0x88, 0x3D, 0x20, 
	0x39, 0x18, 0x77, 0x64, 0xA9, 0x56, 0xAB, 0x21, 0x53, 0xE4, 0xE0, 0x08, 0x72, 0xAC, 0xDB, 0x20, 
	0x27, 0x5D, 0x48, 0x50, 0x78, 0xDB, 0x1F, 0x97, 0x67, 0x89, 0x1A, 0x40, 0x45, 0x02, 0x0F, 0x79, 
	0x03, 0x1C, 0x80, 0xA7, 0x32, 0x14, 0x0C, 0x19, 0x6E, 0xC6, 0x8C, 0x71, 0x15, 0x30, 0x29, 0x18, 
	0x7B, 0x39, 0xF1, 0xDE, 0x1D, 0x22, 0x1D, 0x98, 0x99, 0x1A, 0x0F, 0x18, 0xAC, 0xEA, 0x5D, 0x58, 
	0x0B, 0x03, 0xA3, 0xAA, 0x22, 0x3F, 0x0A, 0xF0, 0xFE, 0x35, 0x49, 0xCD, 0xFA, 0x7B, 0x0C, 0x7A, 
	0xBE, 0xC0, 0xB2, 0xA2, 0x99, 0x73, 0x2A, 0xB7, 0x39, 0xC2, 0xB9, 0x08, 0x4B, 0xB1, 0xC9, 0x41, 
	0x8C, 0xBF, 0x8C, 0xA1, 0x7F, 0x30, 0x2C, 0xCD, 0x9C, 0x66, 0x19, 0xAC, 0x19, 0x53, 0x83, 0xC5, 
	0xA0, 0x16, 0xE5, 0xB9, 0x43, 0x3D, 0x3B, 0x99, 0x36, 0xB0, 0x75, 0x2F, 0x15, 0x58, 0x6E, 0xBD, 
	0xCE, 0x59, 0xF8, 0xBA, 0x70, 0x47, 0xF6, 0x7E, 0x36, 0x3D, 0x01, 0xA4, 0x97, 0x4C, 0x9B, 0x1A, 
	0x8D, 0x58, 0xF2, 0x60, 0x51, 0x41, 0x43, 0x50, 0xAC, 0x1B, 0xEF, 0xDE, 0xDA, 0xAC, 0x2A, 0x0E, 
	0x3A, 0x1D, 0xE8, 0x18, 0x74, 0x9D, 0xB7, 0xFE, 0x8B, 0x1F, 0xB1, 0x0D, 0xC1, 0x43, 0xD4, 0x06, 
	0x65, 0x07, 0xFA, 0x79, 0xC0, 0xC2, 0x43, 0x9E, 0xF9, 0x97, 0x79, 0xCC, 0xD6, 0xEA, 0x26, 0xC0, 
	0x3C, 0x72, 0xAD, 0x1D, 0xD0, 0xA7, 0x5C, 0x84, 0xC0, 0x09, 0x16, 0x58, 0x76, 0x8A, 0xD3, 0x6F, 
	0xB4, 0x5A, 0xEB, 0x78, 0xEB, 0xE7, 0x3A, 0xF1, 0x00, 0xD5, 0x00, 0x91, 0x4E, 0xBB, 0x86, 0x21, 
	0x00, 0x9E, 0xDB, 0x92, 0x33, 0x48, 0x9B, 0xEB, 0xC3, 0xCF, 0x9C, 0xB3, 0xAD, 0x6C, 0xA4, 0x4C, 
	0x64, 0xD8, 0xD5, 0x34, 0xBA, 0x5C, 0xD0, 0xED, 0x63, 0xAC, 0x55, 0x0E, 0x37, 0x33, 0xED, 0x06, 
	0x9C, 0x7E, 0xA4, 0xC4, 0xCF, 0x01, 0x53, 0x05, 0x3C, 0x95, 0x40, 0xE5, 0xF3, 0xF1, 0x04, 0xF5, 
	0x4B, 0x16, 0x03, 0x00, 0x7A, 0x30, 0xD0, 0x0A, 0xF8, 0xF7, 0xA1, 0xB6, 0xFA, 0x28, 0xB4, 0xBC, 
	0x2D, 0x0D, 0x27, 0x02, 0x5B, 0xC5, 0xF9, 0x0D, 0xC4, 0xA6, 0x7A, 0xFE, 0x28, 0x18, 0x79, 0xC1, 
	0xA7, 0x94, 0x2B, 0x89, 0x1A, 0x5F, 0x68, 0x3B, 0x1B, 0x9E, 0xA6, 0x40, 0x5F, 0xC9, 0xE6, 0x18, 
	0x51, 0x5A, 0x76, 0x2E, 0x2D, 0x23, 0xA9, 0x73, 0x2B, 0x34, 0x6C, 0xA0, 0xC7, 0x85, 0x76, 0x8D, 
	0x77, 0xF4, 0xC5, 0xEF, 0x64, 0xB1, 0x55, 0x95, 0x8B, 0x86, 0x2B, 0xC3, 0xC1, 0x5B, 0xDA, 0xA6, 
	0x8B, 0x06, 0xE1, 0x77, 0x6E, 0x31, 0x10, 0x8B, 0xB4, 0x7D, 0x5E, 0x01, 0x47, 0x99, 0xBF, 0x3B, 
	0xB0, 0x75, 0x56, 0x94, 0x19, 0xA5, 0x8A, 0x90, 0x86, 0x5D, 0x90, 0x10, 0xA8, 0x4F, 0x93, 0xD5, 
	0x9C, 0xF3, 0xB1, 0xCE, 0xB5, 0x87, 0xC4, 0x10, 0xCD, 0xD1, 0x1D, 0x1E, 0xCD, 0x0F, 0xB9, 0x29, 
	0xD5, 0x6E, 0xFB, 0x82, 0xC7, 0xC3, 0x3A, 0xDC, 0x5C, 0xE2, 0x3A, 0x85, 0x9A, 0xCE, 0x4F, 0x0B, 
	0x6F, 0x2A, 0x15, 0x9D, 0x30, 0x21, 0x6B, 0xAE, 0x13, 0x0B, 0x2D, 0x0B, 0x8A, 0x12, 0xC7, 0x07, 
	0x83, 0xA2, 0x12, 0x48, 0x9F, 0x78, 0xF1, 0x1A, 0x24, 0x0A, 0xC1, 0xBE, 0xD6, 0x95, 0xBE, 0xE7, 
	0x9B, 0x3F, 0x60, 0xF6, 0x8A, 0xB5, 0x0C, 0xA5, 0xED, 0x3A, 0x99, 0x75, 0xC8, 0x1E, 0x23, 0x7C, 
	0x36, 0x7A, 0x15, 0x96, 0x44, 0x54, 0x8D, 0x3D, 0x6E, 0x47, 0x34, 0x2D, 0x1A, 0x84, 0xDC, 0xE2, 
	0xA8, 0xBB, 0xBC, 0xF1, 0xCB, 0x34, 0xFB, 0x4E, 0x01, 0x4C, 0x80, 0x17, 0x8C, 0xC9, 0xB1, 0x67, 
	0xC7, 0xD0, 0xF9, 0xF7, 0x86, 0xBD, 0xDC, 0xF7, 0xE5, 0x15, 0x18, 0x89, 0xE0, 0xD0, 0x2B, 0x97, 
	0x86, 0x52, 0x59, 0xD7, 0x78, 0x5D, 0xF2, 0x42, 0x8B, 0x47, 0x4A, 0x6D, 0x04, 0xD5, 0x35, 0x35, 
	0xCC, 0xD9, 0x8F, 0x86, 0x3F, 0x27, 0x38, 0x29, 0xEF, 0xBD, 0x8E, 0x62, 0x88, 0xB2, 0x3C, 0x51, 
	0xC9, 0x39, 0x4F, 0x8A, 0x3C, 0xF1, 0x9D, 0xB8, 0xCA, 0x75, 0x77, 0x3C, 0xAA, 0xF5, 0x65, 0x28, 
	0xA6, 0xFD, 0x23, 0x2C, 0x7C, 0x7E, 0x4F, 0x8F, 0xE2, 0x9B, 0x90, 0xFF, 0x64, 0xEA, 0x6E, 0xB1, 
	0xF6, 0xB3, 0x32, 0x4C, 0x65, 0xFF, 0xD1, 0x84, 0x1A, 0x94, 0x52, 0xDA, 0xA8, 0x6C, 0x82, 0xF1, 
	0x78, 0x66, 0x76, 0xF3, 0xBA, 0x69, 0xED, 0x9A, 0x75, 0x5D, 0x9F, 0xC1, 0x11, 0xDA, 0x44, 0xE5, 
	0xE1, 0x39, 0x11, 0x68, 0x5D, 0x9C, 0x67, 0x01, 0x49, 0x6E, 0xD1, 0x39, 0x2B, 0xEB, 0xEC, 0xD7, 
	0x39, 0x3E, 0x26, 0xF4, 0xF0, 0xD5, 0x3D, 0x83, 0x8C, 0x08, 0x49, 0x49, 0x07, 0x77, 0x60, 0xE2, 
	0xA0, 0x7C, 0xE9, 0xD5, 0x60, 0xBE, 0xFA, 0x4D, 0xE2, 0x91, 0x52, 0xBA, 0x8C, 0x6F, 0x1B, 0xEB, 
	0xD9, 0x23, 0x6B, 0xFD, 0x00, 0x1B, 0xB3, 0xA3, 0x6C, 0x6F, 0x15, 0x0D, 0xCF, 0x95, 0x34, 0xD6, 
	0xBE, 0x81, 0xD7, 0xC9, 0x9A, 0xD9, 0xE7, 0x20, 0x9A, 0x68, 0xD6, 0xDC, 0x16, 0x1F, 0x14, 0xD8, 
	0xA4, 0x56, 0xB2, 0xA5, 0xF0, 0x01, 0x04, 0x54, 0x0A, 0x9E, 0x0F, 0xD4, 0x67, 0x88, 0x4A, 0x33, 
	0x13, 0x37, 0x25, 0x19, 0xF6, 0x72, 0x08, 0x7B, 0x70, 0xCC, 0xB3, 0x37, 0xD5, 0x1D, 0x65, 0xAF, 
	0xD5, 0x0A, 0xFF, 0x2F, 0x5B, 0xFB, 0x8C, 0x00, 0xBF, 0x43, 0x0A, 0x00, 0xFB, 0x2F, 0x87, 0xE3, 
	0xB5, 0xE1, 0x25, 0xF0, 0xDF, 0x5A, 0xE5, 0x17, 0x20, 0x28, 0xE4, 0x96, 0xFD, 0xC1, 0x20, 0x16, 
	0x0E, 0xDC, 0x83, 0x8B, 0xDE, 0x12, 0x13, 0xE6, 0x3A, 0x87, 0x9E, 0x57, 0x97, 0xC5, 0x60, 0x96, 
	0x20, 0x9E, 0x6D, 0xA4, 0xAA, 0x0A, 0x47, 0x3B, 0x0B, 0x5C, 0x19, 0xCC, 0x3B, 0x40, 0x50, 0x25, 
	0x1A, 0xA6, 0x40, 0xCE, 0x6B, 0xC3, 0x6A, 0x2D, 0x0F, 0x09, 0xEB, 0xA0, 0xC9, 0xB7, 0xF5, 0xAF, 
	0x6D, 0x1C, 0x0A, 0x77, 0xA1, 0x43, 0x6E, 0xC3, 0xE5, 0x68, 0x9B, 0x99, 0x47, 0x75, 0x48, 0x09, 
	0x83, 0xC7, 0xFB, 0xDA, 0x2C, 0xCB, 0x85, 0x00, 0xB7, 0xB0, 0x79, 0x15, 0xB3, 0x70, 0xC6, 0x99, 
	0x96, 0xE2, 0xFE, 0x9F, 0xE5, 0x39, 0x86, 0xAB, 0x05, 0x9F, 0xEE, 0xBE, 0xAC, 0xF2, 0x0E, 0xDA, 
	0xDA, 0xD5, 0x2C, 0xBF, 0x92, 0xC4, 0xD5, 0xCA, 0xA9, 0x78, 0xBB, 0xB4, 0x95, 0x61, 0xDB, 0xDB, 
	0xA4, 0x58, 0xD7, 0x6D, 0xFE, 0xE2, 0xEC, 0x47, 0x0A, 0x4D, 0xA4, 0xB2, 0x20, 0x9F, 0x4C, 0xFE, 
	0x13, 0x4F, 0x5A, 0x79, 0xF7, 0x95, 0xF3, 0xBC, 0x49, 0x93, 0x2A, 0x6A, 0x05, 0x1C, 0xBE, 0xEE, 
	0xE4, 0xB7, 0x19, 0x03, 0x98, 0xCA, 0x4B, 0xDE, 0x4E, 0x50, 0xEE, 0xD1, 0x9C, 0xD8, 0xE1, 0x83, 
	0xCD, 0xAA, 0x2C, 0x43, 0xFF, 0x40, 0xA0, 0x43, 0x6E, 0x35, 0xD8, 0xD0, 0xB3, 0xF9, 0x9E, 0x5D, 
	0x14, 0x38, 0x31, 0x48, 0x82, 0x1B, 0x00, 0xD6, 0xBB, 0x5F, 0xC6, 0x73, 0xCF, 0x51, 0xA0, 0xFF, 
	0xB0, 0x08, 0xC2, 0x18, 0xF9, 0xE6, 0x17, 0xF4, 0x5F, 0x58, 0xC3, 0x48, 0x34, 0x00, 0xE2, 0x7E, 
	0xCA, 0x6C, 0x6B, 0xD9, 0x27, 0xE1, 0x4D, 0x3C, 0x7E, 0xDE, 0x2D, 0x97, 0xE8, 0x0E, 0xFF, 0xDC, 
	0x02, 0x55, 0x53, 0x1F, 0xEB, 0x86, 0xD0, 0xAB, 0x50, 0x1F, 0xA1, 0x23, 0xB8, 0x00, 0x27, 0x3F, 
	0xD0, 0x99, 0xE3, 0xC9, 0x5D, 0x84, 0x62, 0xEC, 0xC5, 0x47, 0xA1, 0xF5, 0xC8, 0x5B, 0x68, 0x9E, 
	0xB9, 0x6F, 0xF0, 0x83, 0x87, 0x24, 0x3D, 0xFD, 0x52, 0x2B, 0x84, 0xFC, 0x1B, 0xE2, 0x53, 0xA0, 
	0x94, 0x26, 0xF0, 0x3C, 0x0B, 0x09, 0x17, 0x2B, 0x6E, 0x23, 0x4E, 0x50, 0x67, 0x7C, 0x2E, 0x34, 
	0x13, 0xC0, 0xFC, 0x5A, 0x06, 0x04, 0x87, 0x1D, 0xC3, 0x2A, 0x0F, 0xE1, 0x11, 0x72, 0xE0, 0xE1, 
	0xE5, 0xC8, 0x79, 0xB3, 0x6B, 0x77, 0xB9, 0x29, 0x98, 0x5E, 0xE0, 0x96, 0x1C, 0x3E, 0x21, 0x88, 
	0x19, 0x94, 0x53, 0xCA, 0x54, 0xFA, 0x9B, 0xCE, 0x53, 0x7A, 0x89, 0x73, 0x10, 0x79, 0x39, 0x27, 
	0x8C, 0xCF, 0x13, 0x3B, 0x9F, 0x70, 0xC7, 0x97, 0xF7, 0x4A, 0x53, 0x39, 0xFF, 0xE6, 0x13, 0xCC, 
	0xE3, 0x3C, 0xF7, 0xC4, 0x98, 0xAD, 0xF5, 0x84, 0x68, 0x7B, 0xA1, 0x75, 0x64, 0x24, 0x6E, 0x93, 
	0xDA, 0xF5, 0x9A, 0x15, 0xE6, 0x08, 0x2D, 0xC3, 0x2E, 0x4B, 0xBA, 0x97, 0x79, 0x1A, 0x4E, 0xC2, 
	0x96, 0xD5, 0x67, 0xC2, 0x74, 0xE0, 0x7A, 0x3D, 0x44, 0x95, 0x11, 0x5B, 0x72, 0xDF, 0x12, 0x80, 
	0x83, 0xB2, 0xF1, 0xBD, 0xAE, 0x66, 0xF5, 0xD0, 0x1D, 0xC2, 0x4B, 0x2B, 0x05, 0x97, 0xAD, 0x28, 
	0xD1, 0x3C, 0x53, 0x90, 0xF4, 0x67, 0xE5, 0x30, 0x4C, 0x57, 0x76, 0xF1, 0xF6, 0xC4, 0x0F, 0x56, 
	0x27, 0xEC, 0xC1, 0x2D, 0x49, 0x7A, 0xD8, 0x5C, 0xFD, 0xC0, 0xD5, 0x39, 0x73, 0xE8, 0x1C, 0x77, 
	0x1C, 0x3B, 0x87, 0xB3, 0xFE, 0x8E, 0xFB, 0x24, 0xE8, 0xBA, 0x40, 0x57, 0xC4, 0xA6, 0x92, 0xD9, 
	0x18, 0x28, 0x3D, 0xAA, 0x7B, 0x60, 0xAB, 0xEB, 0x56, 0x05, 0x82, 0x6F, 0xA0, 0xD5, 0xAB, 0xD8, 
	0xF3, 0x94, 0x18, 0x27, 0x51, 0xF5, 0x5F, 0xF9, 0x1C, 0x2A, 0x6D, 0x1D, 0x17, 0x6E, 0xA2, 0xA8, 
	0x46, 0x55, 0xC7, 0x6C, 0x21, 0x88, 0xE8, 0x8B, 0xA1, 0xCA, 0x2A, 0xC6, 0x33, 0xFD, 0xCB, 0x0E, 
	0x54, 0xEB, 0x4F, 0xAF, 0x72, 0x99, 0x24, 0x9B, 0x45, 0xE9, 0x9E, 0x22, 0x17, 0x9A, 0xDF, 0x96, 
	0x60, 0x45, 0xD9, 0x03, 0xA5, 0x10, 0xEC, 0x0E, 0xA0, 0x04, 0xCE, 0x70, 0xA9, 0x5B, 0x73, 0x0E, 
	0xE1, 0x0B, 0xB4, 0xC8, 0x09, 0xFC, 0xB8, 0x69, 0x90, 0xA5, 0x55, 0x17, 0x15, 0x25, 0xE0, 0xF2, 
	0xF3, 0x82, 0xED, 0x4D, 0x58, 0x78, 0xEE, 0x76, 0x43, 0xD1, 0xCA, 0x77, 0x0C, 0x2F, 0x9B, 0x65, 
	0x50, 0xBF, 0x85, 0xD2, 0xAC, 0xED, 0x3F, 0x01, 0x42, 0xE4, 0xE1, 0xE4, 0xFC, 0xEB, 0xFD, 0x67, 
	0x3D, 0xAA, 0x31, 0x92, 0xFB, 0x00, 0x08, 0x68, 0xA5, 0xBA, 0x9C, 0x08, 0xFC, 0xA6, 0x0D, 0x87, 
	0xAC, 0x99, 0x9D, 0x1F, 0xA4, 0xFE, 0x05, 0xE5, 0x6C, 0xFB, 0x6C, 0xDD, 0xA1, 0xC5, 0xF8, 0x96, 
	0x2A, 0x4E, 0x0E, 0xE7, 0xD1, 0x0E, 0x8F, 0x88, 0xEC, 0x01, 0x88, 0xF3, 0xB6, 0xEA, 0x3D, 0x3D, 
	0xB5, 0x57, 0xF7, 0x70, 0x5E, 0x88, 0xC2, 0xBB, 0x90, 0x5A, 0x05, 0x9B, 0xDE, 0xB7, 0xF1, 0x27, 
	0x3B, 0x9B, 0xD2, 0xF7, 0x32, 0x2D, 0xEF, 0x12, 0x87, 0x50, 0xF6, 0x24, 0x57, 0xA2, 0x92, 0xDF, 
	0x6D, 0xD7, 0x2B, 0xB3, 0x4B, 0x7C, 0xE7, 0x7E, 0xEE, 0x2E, 0x34, 0xD2, 0x69, 0x4A, 0xCC, 0x64, 
	0x4C, 0x3D, 0xD2, 0x1D, 0x7A, 0xAC, 0xC2, 0xA0, 0x4F, 0x04, 0x8F, 0x90, 0xA2, 0x1E, 0xF0, 0xD3, 
	0x46, 0x29, 0xC7, 0x66, 0x04, 0xB0, 0xB9, 0xF5, 0xDF, 0xA9, 0x84, 0x6D, 0x4D, 0x74, 0xC5, 0xDC, 
	0xF0, 0x14, 0x32, 0x8A, 0xAC, 0xAC, 0x8F, 0xAC, 0xB5, 0x94, 0xDB, 0xF9, 0x01, 0x58, 0x69, 0xBC, 
	0xA9, 0x1D, 0x51, 0xA0, 0x99, 0xD8, 0x21, 0x4A, 0x2D, 0xE2, 0x18, 0x7C, 0x67, 0xAF, 0x63, 0xDF, 
	0x8B, 0xB8, 0x17, 0x9A, 0x41, 0x4A, 0x76, 0x18, 0xEC, 0xF0, 0x8B, 0xD0, 0x32, 0x06, 0x65, 0x0B, 
	0xD6, 0xA4, 0x42, 0x8A, 0xE5, 0xE9, 0xCA, 0x3F, 0x70, 0x7E, 0xA7, 0x8F, 0xB8, 0xF3, 0x49, 0xA9, 
	0xF8, 0x73, 0x75, 0xAE, 0x87, 0xF9, 0x38, 0xE2, 0xE8, 0x87, 0x5B, 0x88, 0x0D, 0x1F, 0x23, 0xE8, 
	0x73, 0xF4, 0x55, 0xE0, 0xE9, 0x39, 0xEC, 0x94, 0xE5, 0x2F, 0xB5, 0xB0, 0xF8, 0x36, 0x55, 0x8B, 
	0xD3, 0xA4, 0x9E, 0x71, 0x1C, 0x86, 0x63, 0xA0, 0xC3, 0x21, 0xDC, 0x1F, 0x8B, 0x34, 0x98, 0xE2, 
	0xDF, 0xFE, 0xAB, 0x7A, 0x1A, 0xE1, 0xF7, 0xB0, 0xE2, 0x33, 0x2E, 0xC6, 0x2E, 0x5E, 0x91, 0xB3, 
	0xF6, 0x2E, 0xAF, 0xF1, 0xC2, 0xE6, 0xE3, 0xFE, 0xB9, 0xF1, 0x3E, 0x98, 0x89, 0x5E, 0x19, 0xF7, 
	0xC7, 0x2C, 0x58, 0x21, 0xFB, 0x10, 0x27, 0x20, 0x2D, 0xCE, 0x32, 0x67, 0xB6, 0xBC, 0xC7, 0x9F, 
	0x3E, 0x06, 0xA5, 0x9F, 0x28, 0xC6, 0xA5, 0xCB, 0xAC, 0x75, 0x12, 0xFA, 0x2B, 0x96, 0x81, 0x87, 
	0x15, 0xF1, 0x8A, 0x4E, 0xAC, 0x60, 0xA6, 0x01, 0x34, 0xF8, 0xB2, 0xB6, 0x4E, 0xBC, 0x26, 0x71, 
	0x7D, 0xEF, 0x17, 0x15, 0x6B, 0xC4, 0x56, 0xF1, 0x18, 0x6F, 0xC9, 0x28, 0xF7, 0x68, 0x7A, 0x93, 
	0x61, 0x6C, 0xD1, 0xF0, 0x52, 0xC0, 0xBC, 0x38, 0x27, 0x7F, 0x4A, 0x2D, 0x6A, 0x8F, 0x09, 0xFE, 
	0xCD, 0xA3, 0x6B, 0xCF, 0x36, 0xE0, 0xD6, 0x9F, 0x4C, 0xF9, 0xD8, 0x58, 0x94, 0x53, 0x2A, 0xE8, 
	0x7A, 0x5F, 0xA3, 0xB6, 0x89, 0x3D, 0x48, 0xBD, 0x4B, 0x23, 0xDC, 0xE1, 0xE6, 0x6A, 0xCE, 0x88, 
	0x13, 0x14, 0x90, 0x3B, 0xDA, 0xD0, 0x84, 0xCC, 0x59, 0x1D, 0x6C, 0xB4, 0x99, 0x69, 0x7A, 0x7E, 
	0x20, 0x35, 0x83, 0x41, 0xE9, 0xB7, 0x7C, 0xD8, 0x4C, 0xF2, 0x5E, 0x02, 0x1D, 0xC6, 0x4B, 0xA5, 
	0xAF, 0x5B, 0xF2, 0x00, 0x52, 0xA2, 0xCA, 0x3A, 0xAF, 0xFB, 0x00, 0xE8, 0xF1, 0x44, 0xA5, 0x45, 
	0x0D, 0xD5, 0xA9, 0x08, 0x2A, 0x64, 0xAD, 0x13, 0xD5, 0x0C, 0xB4, 0xBD, 0x4D, 0x55, 0xCA, 0xC3, 
	0x46, 0xF1, 0x8F, 0x41, 0xBD, 0x6B, 0x6B, 0x1E, 0xF5, 0xB1, 0x80, 0x4F, 0xE0, 0x35, 0x48, 0x28, 
	0xAA, 0x6A, 0x69, 0x0F, 0xA9, 0x49, 0x07, 0xDB, 0x53, 0xE9, 0xDF, 0xFD, 0xD4, 0x8F, 0x74, 0xBC, 
	0x01, 0x71, 0x30, 0x69, 0x33, 0x51, 0xDD, 0x2F, 0x44, 0x1F, 0xB0, 0x8D, 0xDD, 0x21, 0xD2, 0x3B, 
	0xE0, 0xCA, 0xF0, 0x2F, 0xD8, 0xA4, 0xB4, 0x39, 0x5E, 0x11, 0x86, 0x98, 0xCE, 0xAD, 0xC8, 0x21, 
	0x85, 0x23, 0x30, 0x8B, 0x6C, 0x8B, 0xE8, 0x81, 0x5E, 0x05, 0xBE, 0xF2, 0xFC, 0x72, 0x7D, 0x9A, 
	0x21, 0x51, 0x19, 0xDC, 0x38, 0x1D, 0xE6, 0x71, 0xEE, 0x4E, 0x7F, 0x8C, 0x14, 0xCF, 0x4C, 0x30, 
	0x27, 0x6F, 0xB5, 0x1F, 0xFB, 0xA0, 0x93, 0x66, 0xD8, 0xCC, 0xDA, 0x25, 0x3B, 0xA9, 0x9B, 0x1B, 
	0x46, 0x6C, 0x15, 0x19, 0x5C, 0x5B, 0xCF, 0xF4, 0xD2, 0x17, 0xF8, 0xB9, 0x1C, 0x0C, 0x14, 0xF7, 
	0xA4, 0x92, 0x46, 0xBE, 0xE4, 0x4A, 0x74, 0x83, 0xAC, 0x81, 0x1E, 0x2A, 0xB9, 0x67, 0xD2, 0xAF, 
	0x53, 0x2E, 0xD3, 0xAE, 0x10, 0xF3, 0xA6, 0x36, 0xEB, 0x3F, 0x76, 0xFE, 0x89, 0x1B, 0xDB, 0x30, 
	0x7F, 0xBA, 0x61, 0xAC, 0x45, 0xF2, 0x20, 0xA8, 0x6C, 0x1A, 0x19, 0x2A, 0x78, 0x64, 0x57, 0xB9, 
	0x7D, 0x13, 0x97, 0xE4, 0x6F, 0x7B, 0x1F, 0xD0, 0xC5, 0xE1, 0x62, 0xC5, 0xA8, 0x8E, 0x2A, 0x3C, 
	0x2D, 0xB0, 0xF8, 0xB4, 0xA1, 0x2A, 0xB3, 0xDE, 0xF2, 0x15, 0x67, 0xA7, 0x5B, 0xBC, 0xA1, 0x58, 
	0x71, 0x8A, 0x3B, 0x73, 0xF3, 0x49, 0xED, 0x99, 0x95, 0x32, 0x6F, 0xC4, 0x53, 0xED, 0x5C, 0xE5, 
	0xE5, 0xE7, 0x0D, 0x52, 0x6E, 0x72, 0x46, 0x28, 0x57, 0x3D, 0x64, 0xB5, 0xCA, 0xD6, 0x04, 0x03, 
	0x43, 0x62, 0xEC, 0xDB, 0x57, 0x5E, 0xCE, 0xDE, 0xFF, 0xE5, 0x38, 0x74, 0xD2, 0x44, 0x6B, 0x34, 
	0x8D, 0x98, 0x72, 0xEB, 0xC2, 0x9D, 0x43, 0xD7, 0xC1, 0xEA, 0x1E, 0x6E, 0x3E, 0x57, 0x4A, 0x70, 
	0x4E, 0x7E, 0x15, 0xF8, 0x24, 0xC0, 0xDD, 0x48, 0xE6, 0xF4, 0x39, 0x18, 0x3A, 0x8B, 0x12, 0x1D, 
	0xB4, 0x7C, 0xAD, 0x31, 0x19, 0xDE, 0xC0, 0xFA, 0x50, 0x23, 0x1E, 0xD8, 0x6D, 0xE3, 0x75, 0x09, 
	0x79, 0xED, 0x75, 0x1F, 0x7A, 0x52, 0x24, 0xF2, 0xD3, 0xF8, 0xA8, 0xFE, 0x55, 0x1E, 0xD8, 0x6F, 
	0xA5, 0x19, 0x9D, 0xFE, 0x08, 0x54, 0x20, 0x1E, 0xA0, 0x21, 0xB8, 0xE3, 0x3E, 0xE3, 0x35, 0x34, 
	0xF4, 0x8D, 0xA8, 0xC3, 0x0E, 0xA9, 0x2E, 0xC0, 0xD2, 0x31, 0xD2, 0x1C, 0x23, 0x00, 0x65, 0x48
};

int _kprx_decrypt(u8 *prx, u32 size, u32 *newsize, u32 use_polling)
{
	int ret;
	u32 type = 0, scramble_code = 0, i=0;
	u8 *key_addr = NULL;
	u8 *blacklist = NULL;
	u32 blacklistsize = 0;

	// we use 6.20 key for now
	memcpy(g_kernel_phat_key, keys620_0, sizeof(keys620_0));
	memcpy(g_kernel_slim_key, keys620_1, sizeof(keys620_1));

	if (prx == NULL || newsize == NULL) {
		ret = -201;
		goto exit;
	}

	if (size < 0x160) {
		ret = -202;
		goto exit;
	}

	if ((u32)prx & 0x3f) {
		ret = -203;
		goto exit;
	}

	if (!((0x00220202 >> ((((u32)prx) >> 27) & 0x0000001F)) & 1)) {
		ret = -204;
		goto exit;
	}

	memcpy(buf_3, prx, 0x150);
	
	if (0x4C941DF0 == *(u32*)&buf_3[0xd0]) {
        key_addr = g_kernel_slim_key;
		scramble_code = 0x43;
		type = 2;
	} else if (0x4C940FF0 == *(u32*)&buf_3[0xd0]) {
		key_addr = key_2DA8;
		scramble_code = 0x43;
		type = 2;
	} else if (0x4C941CF0 == *(u32*)&buf_3[0xd0]) {
		key_addr = g_kernel_phat_key;
		scramble_code = 0x43;
		type = 2;
	} else if (0x4C940AF0 == *(u32*)&buf_3[0xd0]) {
		key_addr = key_2DB8;
		scramble_code = 0x43;
		type = 2;
	} else if (0xCFEF08F0 == *(u32*)&buf_3[0xd0]) {
		key_addr = key_2D90;
		scramble_code = 0x62;
		type = 2;
	} else if (0xCFEF06F0 == *(u32*)&buf_3[0xd0]) {
		key_addr = key_2D80;
		scramble_code = 0x62;
		type = 2;
	} else if (0xCFEF05F0 == *(u32*)&buf_3[0xd0]) {
		key_addr = key_2380;
		scramble_code = 0x62;
		type = 2;
	} else if (0x16D59E03 == *(u32*)&buf_3[0xd0]) {
		key_addr = key_2370;
		scramble_code = 0x62;
		type = 2;
	} else if (0x4467415D == *(u32*)&buf_3[0xd0]) {
		key_addr = key_22E0;
		scramble_code = 0x59;
		type = 1;
	} else if (0x00000000 == *(u32*)&buf_3[0xd0]) {
		key_addr = key_21C0;
		blacklist = blacklist_data;
		blacklistsize = 0x970;
		scramble_code = 0x42;
		type = 0;
	} else if (0x01000000 == *(u32*)&buf_3[0xd0]) {
		key_addr = key_2250;
		scramble_code = 0x43;
		type = 0;
	} else {
		ret = -301;
		goto exit;
	}

	if (type == 2) {
		u8 *p = buf_3;

		i = 0;
		while (i<0x58) {
			if (p[0xd4]) {
				ret = -302;
				goto exit;
			}

			i++;
			p++;
		}
	}

	// you may disable module blacklist check here
	ret = check_blacklist(prx, blacklist, blacklistsize);

	if (ret == 1) {
		ret = -305;
		goto exit;
	}

	*newsize = *(u32*)&prx[0xb0];

	if (size - 0x150 < *newsize) {
		ret = -206;
		goto exit;
	}

	if (type == 2) {
		int i;

		for (i=0; i<9; i++)
		{
			memcpy(buf_1 + 0x14 + (i << 4), key_addr, 0x10);
			buf_1[0x14+ (i<<4)] = i;
		}

		memset(g_kernel_phat_key, 0, 0x10);
		memset(g_kernel_slim_key, 0, 0x10);
	} else {
		memcpy(buf_1+20, key_addr, 0x90);
	}

	ret = kirk7(buf_1, 0x90, scramble_code, use_polling);

	if (ret != 0) {
		if (ret < 0) {
			ret = -101;
		} else {
			if (ret == 0xc) {
				ret = -107;
			} else {
				ret = -104;
			}
		}

		goto exit;
	}

	memcpy(buf_4, buf_1, 0x90);

	if (type == 2) {
		memcpy(buf_1, &buf_3[0xd0], 0x5c);
		memcpy(buf_1+0x5c, &buf_3[0x140], 0x10);
		memcpy(buf_1+0x5c+0x10, &buf_3[0x12c], 0x14);
		memcpy(buf_1+0x5c+0x10+0x14, &buf_3[0x80], 0x30);
		memcpy(buf_1+0x5c+0x10+0x14+0x30, &buf_3[0xc0], 0x10);
		memcpy(buf_1+0x5c+0x10+0x14+0x30+0x10, &buf_3[0xb0], 0x10);
		memcpy(buf_1+0x5c+0x10+0x14+0x30+0x10+0x10, &buf_3[0], 0x80);
	} else {
		memcpy(buf_1, &buf_3[0xd0], 0x80);
		memcpy(buf_1+0x80, &buf_3[0x80], 0x50);
		memcpy(buf_1+0x80+0x50, &buf_3[0], 0x80);
	}

	if (type == 1) {
		memcpy(buf_2+0x14, buf_1+0x10, 0xa0);
		ret = kirk7(buf_2, 0xa0, scramble_code, use_polling);

		if (ret != 0) {
			if (ret >= 0) {
				if (ret == 0xc)
					ret = -107;
				else
					ret = -106;
			} else {
				ret = -103;
			}

			memcpy(prx, buf_3, 0x150);
			goto exit;
		}

		memcpy(buf_1+0x10, buf_2, 0xa0);
	}

	if (type == 2) {
		memcpy(buf_2+20, buf_1+92, 0x60);
		ret = kirk7(buf_2, 0x60, scramble_code, use_polling);

		if (ret != 0) {
			if (ret >= 0) {
				if (ret == 0xc)
					ret = -107;
				else
					ret = -106;
			} else {
				ret = -103;
			}

			memcpy(prx, buf_3, 0x150);
			goto exit;
		}

		memcpy(buf_1+92, buf_2, 0x60);
	}

	if (type == 2) {
		memcpy(buf_2, buf_1+108, 0x14);
		memcpy(buf_1+112, buf_1+92, 0x10);
		memset(buf_1+24, 0, 0x58);
		memcpy(buf_1+4, buf_1, 4);
		*(u32*)buf_1 = 0x14c;
		memcpy(buf_1+8, buf_4, 0x10);
		memset(buf_4, 0, 0x10);
	} else {
		memcpy(buf_2, buf_1+4, 0x14);
		*(u32*)buf_1 = 0x14c;
		memcpy(buf_1+4, buf_4, 0x14);
	}

	if (!use_polling) {
		ret = sceUtilsBufferCopyWithRange (buf_1, 0x150, buf_1, 0x150, 0xB);
	} else {
		ret = sceUtilsBufferCopyByPollingWithRange (buf_1, 0x150, buf_1, 0x150, 0xB);
	}

	if (ret != 0) {
		if (ret > 0) {
			if (ret == 0xc)
				ret = -107;
			else
				ret = -106;
		} else {
			ret = -102;
		}

		goto exit;
	}

	ret = memcmp(buf_1, buf_2, 0x14);

	if (ret != 0) {
		ret = -302;
		goto exit;
	}

	if (type == 2) {
		prx_xor_key_into(buf_1+128, 0x40, buf_1+128, buf_4+16);
		memset(buf_4+16, 0, 0x40);
		ret = kirk7(buf_1+0x6c, 0x40, scramble_code, use_polling);

		if (ret != 0) {
			if (ret > 0) {
				if (ret == 0xc)
					ret = -107;
				else
					ret = -106;
			} else {
				ret = -102;
			}

			goto exit;
		}

		prx_xor_key_into(prx+64, 0x40, buf_1+108, buf_4+80);
		memset(buf_4+80, 0, 0x40);
		memset(prx+128, 0, 0x30);
		prx[160] = 1;
		memcpy(prx+176, buf_1+192, 0x10);
		memset(prx+192, 0, 0x10);
		memcpy(prx+208, buf_1+208, 0x80);
	} else {
		prx_xor_key_into(buf_1+64, 0x70, buf_1+64, buf_4+20);
		ret = kirk7(buf_1+0x2c, 0x70, scramble_code, use_polling);

		if (ret != 0) {
			if (ret > 0) {
				if (ret == 0xc)
					ret = -107;
				else
					ret = -106;
			} else {
				ret = -102;
			}

			goto exit;
		}

		prx_xor_key_into(prx+64, 0x70, buf_1+44, buf_4+32);
		memcpy(prx+176, buf_1+176, 0xa0);
	}

	if (!use_polling) {
		ret = sceUtilsBufferCopyWithRange(prx, size, prx+0x40, size-0x40, 1);
	} else {
		ret = sceUtilsBufferCopyByPollingWithRange(prx, size, prx+0x40, size-0x40, 1);
	}

	if (ret != 0) {
		if (ret > 0) {
			if (ret == 0xc)
				ret = -107;
			else
				ret = -304;
		} else {
			ret = -303;
		}

		goto exit;
	}

	if (*newsize < 0x150) {
		// Fill with 0
		memset(prx+*newsize, 0, 0x150-*newsize);		
	}

	ret = 0;
exit:
	memset(g_kernel_phat_key, 0, 0x10);
	memset(g_kernel_slim_key, 0, 0x10);
	memset(buf_1, 0, 0x150);
	memset(buf_2, 0, 0xb4);
	memset(buf_3, 0, 0x150);
	memset(buf_4, 0, 0x90);

	return ret;
}

/* from 6.20 */
u8 key_23A0[16] = {
	0x3C, 0x28, 0x1E, 0x05, 0x7E, 0x3E, 0xD6, 0x0B,
	0xCB, 0x2C, 0xAC, 0x9F, 0x66, 0xB0, 0xDE, 0x4B
};

/* from 6.20 */
u8 key_2390[16] = {
	0x0F, 0x44, 0x1C, 0x51, 0xEA, 0x9C, 0xF7, 0xB8,
	0xEA, 0x09, 0x95, 0x34, 0x0A, 0xCF, 0x39, 0xE2
};

void memlmd_8450109F(u32 unk, u8 *hash_addr)
{
	u8 *p;
	int i;

	if(unk)
		return;

	p = hash_addr;
	i = 0;

	while(*p++ == '\0' && i < 16)
		i++;

	if (i == 16) {
		return;
	}

	i = 0;

	while (i < 16) {
		g_kernel_phat_key[i] = key_23A0[i] ^ hash_addr[i];
		g_kernel_slim_key[i] = key_2390[i] ^ (hash_addr+16)[i];
		++i;
	}
}

void test_memlmd_8450109F(u8 *result)
{
	u32 k1 = pspSdkGetK1();
	
	pspSdkSetK1(0);
	memset(result, 0, 10);
	memlmd_8450109F(0, (u8*)0xbfc00200);
	memcpy(result, g_kernel_phat_key, 16);
	memcpy(result+16, g_kernel_slim_key, 16);
	pspSdkSetK1(k1);
}

int _kprx_decrypt_(kernel_decryptor *pBlock)
{
	return _kprx_decrypt(pBlock->prx, pBlock->size, pBlock->newsize, pBlock->use_polling);
}

int kprx_decrypt(kernel_decryptor *pBlock)
{
	int ret;
	u32 k1 = pspSdkGetK1();


	pspSdkSetK1(0);
	ret = sceKernelExtendKernelStack(0x2000, (void *)_kprx_decrypt_, pBlock);
	pspSdkSetK1(k1);

	return ret;
}

